@@ -1,31 +1,35 @@
 #' @import curl
 #' @import digest
-#' @import stringr
 #' @import dplyr
-#' @import tidyr
-#' @import purrr
+#' @import htmltools
 #' @import logger
+#' @import purrr
+#' @import rvest
+#' @import stringr
+#' @import tidyr
 #' @import xml2
-#' @importFrom htmltools tags
-#' @importFrom base64enc base64encode
+#' @importFrom base64enc base64decode base64encode
 #' @importFrom commonmark markdown_html
 #' @importFrom glue glue
 #' @importFrom httr http_date
 #' @importFrom mime guess_type
 #' @importFrom rmarkdown render html_document
-#' @importFrom stats setNames
+#' @importFrom stats setNames na.omit
+#' @importFrom stringi stri_replace_all_fixed stri_replace_all_regex
 #' @importFrom tools file_ext
 #' @importFrom urltools url_decode
-#' @importFrom utils packageVersion
-#' @importFrom vctrs new_rcrd vec_recycle_common vec_cast_common vec_assert field
-#' @importFrom vctrs vec_ptype_abbr vec_ptype_full
+#' @importFrom utils packageVersion download.file modifyList
 #' @importFrom xfun read_utf8
 NULL
 
 globalVariables(
   c(
     ".",
+    "e1",
+    "e2",
+    "fingerprint",
     "msg",
-    "runif"
-    )
+    "runif",
+    "detect_language"
   )
+)
