@@ -1,4 +1,4 @@
-new_header <- function(
+header <- function(
   name,
   values,
   sep = NULL
@@ -18,20 +18,27 @@ new_header <- function(
 #' Accepts a header object and formats it as a header field.
 #'
 #' @param x A header object.
+#' @param width The width of the head name field.
 #' @param ... Further arguments passed to or from other methods.
 #' @export
 #'
 #' @return A formatted header field.
-as.character.header <- function(x, ...) {
-  sprintf(
-    "%-26s %s",
-    paste0(x$name, ":"),
+as.character.header <- function(x, width = 30, ...) {
+  FORMAT <- glue("%-{width}s")
+  INDENT <- strrep(" ", width)
+
+  paste0(
+    sprintf(
+        FORMAT,
+        paste0(x$name, ":")
+    ),
     paste(
-      sapply(x$values, as.character),
-      collapse = x$sep
+      as.character(x$values),
+      collapse = paste0(x$sep, CRLF, INDENT)
     )
   )
 }
+# as.character.header(msg$headers$To) %>% cat()
 
 print.header <- function(x, ... ) {
   print(as.character(x))
@@ -44,10 +51,11 @@ header_get <- function(msg, name) {
 header_set <- function(msg, name, values, append = FALSE, sep = NULL) {
   # Get current header.
   header <- msg$headers[[name]]
-  # Header has not previously been set.
   if (is.null(header)) {
-    header <- new_header(name, c(), sep)
+    # Header has not previously been set.
+    header <- header(name, c(), sep)
   } else {
+    # Header has previously been set.
     if (append) {
       values <- c(header$values, values)
     }
