@@ -1,21 +1,28 @@
 test_that("create address", {
   expect_equal(
-    address(
-      c("gerry@gmail.com", "alice@yahoo.com", "jim@aol.com"),
-      c("Gerald", "Alice", NA)
-    ),
-    c(
-      address("gerry@gmail.com", "Gerald"),
-      address("alice@yahoo.com", "Alice"),
-      address("jim@aol.com")
-    )
+    address("gerry@gmail.com") %>% as.character(), "gerry@gmail.com"
+  )
+  expect_equal(
+    address("gerry@gmail.com", NA) %>% as.character(), "gerry@gmail.com"
+  )
+  expect_equal(
+    address("gerry@gmail.com", "Gerald") %>% as.character(), "Gerald <gerry@gmail.com>"
   )
 })
 
 test_that("create address from local and domain", {
   expect_equal(
-    address(local = c("alice", "erin"), domain = "yahoo.com"),
-    address(c("alice@yahoo.com", "erin@yahoo.com"))
+    address(local = "alice", domain = "yahoo.com"),
+    address("alice@yahoo.com")
+  )
+})
+
+test_that("unable to recycle", {
+  expect_error(
+    address(
+      email = c("bob@gmail.com", "alice@yahoo.com"),
+      display = c("Bob", "Alice", "Jim")
+    )
   )
 })
 
@@ -53,14 +60,6 @@ test_that("normalise", {
   expect_equal(as.address("     Gerald    <   gerry@gmail.com    >"), address("gerry@gmail.com", "Gerald"))
 })
 
-test_that("full type of vector", {
-  expect_equal(vec_ptype_full(address("alice@yahoo.com", "Alice")), "address")
-})
-
-test_that("abbreviated type of vector", {
-  expect_equal(vec_ptype_abbr(address("alice@yahoo.com", "Alice")), "addr")
-})
-
 test_that("print address", {
   expect_output(print(address("alice@yahoo.com", "Alice")), "Alice <alice@yahoo.com>")
 })
@@ -68,8 +67,9 @@ test_that("print address", {
 test_that("address operators", {
   expect_true(address("alice@yahoo.com", "Alice") == address("alice@yahoo.com", "Alice"))
   expect_true(address("alice@yahoo.com", "Alice") == "Alice <alice@yahoo.com>")
+  expect_true("Alice <alice@yahoo.com>" == address("alice@yahoo.com", "Alice"))
+  expect_true(address("alice@yahoo.com", "Alice") == address("alice@yahoo.com", "Gerald"))
   expect_true(address("alice@yahoo.com", "Alice") != address("bob@gmail.com", "Bob"))
-  expect_true(address("alice@yahoo.com", "Alice") != address("alice@yahoo.com", "Gerald"))
   expect_true(address("alice@yahoo.com", "Alice") != address("gerry@gmail.com", "Alice"))
   #
   # Undefined operation.
@@ -79,8 +79,8 @@ test_that("address operators", {
 
 test_that("split address list", {
   addr_list <- address(
-    c("gerry@gmail.com", "alice@yahoo.com", "jim@aol.com"),
-    c("Gerald", NA, NA)
+    email = c("gerry@gmail.com", "alice@yahoo.com", "jim@aol.com"),
+    display = c("Gerald", NA, NA)
   )
 
   expect_equal(
@@ -95,4 +95,14 @@ test_that("split address list", {
     as.address(c("Gerald <gerry@gmail.com>", "alice@yahoo.com, jim@aol.com")),
     addr_list
   )
+
+  last_first <- as.address("Durrell, Gerald <gerry@gmail.com>", split = FALSE)
+  expect_equal(
+    last_first,
+    address("gerry@gmail.com", "Durrell, Gerald")
+  )
+  expect_equal(
+    as.character(last_first),
+    '"Durrell, Gerald" <gerry@gmail.com>'
+  )
 })
