@@ -10,12 +10,12 @@ test_that("class envelope", {
 
 test_that("recipient address", {
   recipient <- envelope(to = "bob@gmail.com")
-  expect_equal(recipient$headers$To$values[[1]], address("bob@gmail.com"))
+  expect_equal(recipient$headers$To$values, address("bob@gmail.com"))
 })
 
 test_that("sender address", {
   sender <- envelope(from = "bob@gmail.com")
-  expect_equal(sender$headers$From$values[[1]], address("bob@gmail.com"))
+  expect_equal(sender$headers$From$values, address("bob@gmail.com"))
 })
 
 test_that("maximum one sender address", {
@@ -25,22 +25,22 @@ test_that("maximum one sender address", {
 
 test_that("cc", {
   cc <- envelope(cc = "bob@gmail.com")
-  expect_equal(cc$headers$Cc$values[[1]], address("bob@gmail.com"))
+  expect_equal(cc$headers$Cc$values, address("bob@gmail.com"))
 })
 
 test_that("bcc", {
   bcc <- envelope(bcc = "bob@gmail.com")
-  expect_equal(bcc$headers$Bcc$values[[1]], address("bob@gmail.com"))
+  expect_equal(bcc$headers$Bcc$values, address("bob@gmail.com"))
 })
 
 test_that("reply to", {
   reply <- envelope(reply = "bob@gmail.com")
-  expect_equal(reply$header[["Reply-To"]]$values[[1]], address("bob@gmail.com"))
+  expect_equal(reply$header[["Reply-To"]]$values, address("bob@gmail.com"))
 })
 
 test_that("subject", {
   subject <- envelope(subject = "Email Subject")
-  expect_equal(subject$header$Subject$values[[1]], "Email Subject")
+  expect_equal(subject$header$Subject$values, "Email Subject")
 })
 
 test_that("body text", {
@@ -65,3 +65,16 @@ test_that("parts are not nested", {
     attachment(JPGPATH)
   expect_false(is.nested(msg$parts))
 })
+
+test_that("not splitting addresses", {
+  msg <- envelope(
+    to = c("Durrell, Gerald <gerry@gmail.com>", "Jones, Bob <bob@yahoo.com>"),
+    cc = c("Durrell, Gerald <gerry@gmail.com>", "Jones, Bob <bob@yahoo.com>"),
+    bcc = c("Durrell, Gerald <gerry@gmail.com>", "Jones, Bob <bob@yahoo.com>"),
+    split = FALSE
+  )
+
+  expect_match(headers(msg), 'To:[[:space:]]+"Durrell, Gerald" <gerry@gmail.com>,\r\n[[:space:]]+"Jones, Bob" <bob@yahoo.com>')
+  expect_match(headers(msg), 'Cc:[[:space:]]+"Durrell, Gerald" <gerry@gmail.com>,\r\n[[:space:]]+"Jones, Bob" <bob@yahoo.com>')
+  expect_match(headers(msg), 'Bcc:[[:space:]]+"Durrell, Gerald" <gerry@gmail.com>,\r\n[[:space:]]+"Jones, Bob" <bob@yahoo.com>')
+})
